#!/bin/bash

echo
echo
echo "################################################"
echo "Script by Jesin (http://jesin.tk/)"
echo "################################################"

if [ "$(id -u)" != 0 ]
then
	echo "This script should be run as the root user" 1>&2
exit 1;
fi

echo "Welcome to the unmanaged VPS/Dedicated Server setup script.

Using this script you can:

- update and Upgrade with apt-get
- change your Timezone
- choose locales
- change the SSH server's default port
- disable root user logins via SSH
- create a user with sudo permissions
- create and save iptables firewall rules"

echo

function aptupdate() 
{
	echo -n "Do you want to run apt-get update && apt-get upgrade [y/n]: "
	read aptopt
	if [ $(echo $aptopt | tr '[A-Z]' '[a-z]') = "y" ]
	then
		apt-get update && apt-get upgrade
	elif [ $(echo $aptopt | tr '[A-Z]' '[a-z]') != "n" ]
	then
		aptupdate
	fi
}

function tzconf()
{ 
	echo -n "Do you want to change your timezone ? [y/n] "
	read tzopt
	if [ $(echo $tzopt | tr '[A-Z]' '[a-z]') = "y" ]
	then
		dpkg-reconfigure tzdata
	elif [ $(echo $tzopt | tr '[A-Z]' '[a-z]') != "n" ]
	then
		tzconf
	fi
}

function localesconf()
{
	echo -n "Do you want to configure locales ? [y/n] "
	read locopt
	if [ $(echo $locopt | tr '[A-Z]' '[a-z]') = "y" ]
	then
		dpkg-reconfigure locales
	elif [ $(echo $locopt | tr '[A-Z]' '[a-z]') != "n" ]
	then
		localesconf
	fi
}

function sshportconf()
{
	echo -n "Do you wish to change this ? [y/n] "
	read sshopt
	if [ $(echo $sshopt | tr '[A-Z]' '[a-z]') = "y" ]
	then
		sshchgport
	elif [ $(echo $sshopt | tr '[A-Z]' '[a-z]') != "n" ]
	then
		sshportconf
	fi
}

function sshchgport()
{
	echo -n "Enter the new SSH port no: [49152-65535] "
	read sshdport
	if [ $sshdport -ge 49152 -a $sshdport -le 65535 ]
	then
		sed -i s/"Port $portno"/"Port $sshdport"/g \
		/etc/ssh/sshd_config && \
		echo "" && echo "SSH port changed to $sshdport" && \
		echo "" && flag_sshport=1
	else
		sshchgport
	fi
}

function sshrootlogin()
{
	echo -n "Do you want to disable root user login via SSH ? [y/n] "
	read sshrootopt
	if [ $(echo $sshrootopt | tr '[A-Z]' '[a-z]') = "y" ]
	then
		sed -i 's/PermitRootLogin yes/PermitRootLogin no/g' \
		/etc/ssh/sshd_config && \
		echo "" && echo "Root login via SSH disabled" && \
		echo "" && flag_denyroot=1
	elif [ $(echo $sshrootopt | tr '[A-Z]' '[a-z]') != "n" ]
	then
		sshrootlogin
	fi
}

function sudouser()
{
	echo -n "Do you want to create a user with sudo permissions ? [y/n] "
	read sudoopt
	if [ $(echo $sudoopt | tr '[A-Z]' '[a-z]') = "y" ]
	then
		createsudouser
	elif [ $(echo $sudoopt | tr '[A-Z]' '[a-z]') != "n" ]
	then
		sudouser
	fi
}

function createsudouser()
{
	echo -n "Enter an username: "
	read sudousername
	if [ "$(grep $sudousername: /etc/passwd)" != "" -o \
		"$(grep $sudousername: /etc/group)" != "" ]
	then
		echo "User or Group already exists"
		createsudouser
	else
		echo -n "Enter a strong password for $sudousername :"
		stty -echo
		read sudopasswd
		stty echo
		useradd -m --shell /bin/bash $sudousername && \
		echo $sudousername:$sudopasswd | chpasswd && \
		chmod +w /etc/sudoers && \
		echo "$sudousername	ALL=(ALL) ALL" >> /etc/sudoers && \
		chmod -w /etc/sudoers && \
		echo && \
		echo "The user $sudousername has been successfully created and added to the sudoers list" && flag_sudo=1
	fi
}

function firewallrules()
{
	echo
	echo "NOTE: All your existing firewall rules from the INPUT table will be removed"
	echo -n "Do you want to setup iptables firewall ? [y/n] "
	read firewallopt
	if [ $(echo $firewallopt | tr '[A-Z]' '[a-z]') = "y" ]
	then
		iptables -L MYRULES > /dev/null && \
		iptables -X MYRULES
		iptables -F && \
		iptables -A INPUT -m state --state RELATED,ESTABLISHED \
		-j ACCEPT && \
		iptables -A INPUT -i lo -j ACCEPT && \
		iptables -A INPUT ! -i lo -d 127.0.0.0/8 -j REJECT && \
		iptables -N MYRULES && \
		iptables -A INPUT -j MYRULES && \
		iptables -A MYRULES -m state --state NEW \
		-p tcp --dport 22 -j ACCEPT && \
		if [ "$flag_sshport" = 1 ]
		then
			iptables -A MYRULES -m state --state NEW \
			-p tcp --dport $sshdport -j ACCEPT
		fi && \
		iptables -A MYRULES -m state --state NEW -p tcp --dport 80 \
		-j ACCEPT && \
		iptables -A MYRULES -m state --state NEW -p tcp \
		--dport 443 -j ACCEPT && \
		iptables -A MYRULES -p icmp --icmp-type 8 -j ACCEPT && \
		iptables -A INPUT -j DROP && \
		iptables -A FORWARD -j REJECT
		echo "Firewall rules successfully updated" && \
		iptables-save > /etc/iptables.firewallrules && \
		echo "#!/bin/bash" > /etc/network/if-pre-up.d/firewall && \
		echo "/sbin/iptables-restore < /etc/iptables.firewallrules" >> /etc/network/if-pre-up.d/firewall && \
		chmod +x /etc/network/if-pre-up.d/firewall && \
		echo "Firewall rules successfully written to a file" && \
		echo "If you changed your SSH port you should run thne following commands" && \
		echo && \
		echo "iptables -D MYRULES 1" && \
		echo "iptables-save > /etc/iptables.firewallrules" && \
		echo && \
		echo "Any addition rules can be added with the following command" && \
		echo "iptables -A MYRULES -p <tcp/udp> --dport <portno> -j ACCEPT" && \
		echo
	elif [ $(echo $firewallopt | tr '[A-Z]' '[a-z]') != "n" ]
	then
		firewallrules
	fi
	
}

aptupdate

echo "The current timezone is `date +%Z%:z`"
tzconf

echo
echo "Your current locale is $(locale | grep LANG= | sed 's/LANG=//g')";
localesconf

portno=$(grep Port /etc/ssh/sshd_config | sed 's/Port //g');
if [ $portno = 22 ]
then
	echo
	echo "Your SSH server listens on port number $portno"
	sshportconf
fi

if [ "$(grep "PermitRootLogin yes" /etc/ssh/sshd_config)" != "" ]
then
echo
sshrootlogin
fi

echo
sudouser
firewallrules
